/*global window, rJS, RSVP, Blob, console, URL */
/*jslint indent: 2, maxerr: 3 */
(function (window, rJS, RSVP, jIO) {
  "use strict";

  function getStorageGadget(gadget, new_url) {
    var storage_gadget, site_editor_gadget_url, getURL = window.location;
    return RSVP.Queue()
      .push(function () {
        if (new_url) {
          return new_url;
        } else if (!gadget.props.storage_gadget_url) {
          return gadget.getSetting(
                "site_editor_gadget_url",
                 getURL.protocol + "//" + getURL.host + "/crib-enable.html"
              );
        } else {
          return gadget.props.storage_gadget_url;
        }
      })
      .push(function (url) {
        if (gadget.props.storage_gadget_url == url) {
          return gadget.getDeclaredGadget("storage")
            .push(undefined, function () {
              return gadget.declareGadget(
                url,
                {
                  "scope": "storage",
                  "sandbox": "iframe",
                  "element": gadget.props.element.querySelector('div')
                });
            });
        } else {
          gadget.props.storage_gadget_url = url;
          return gadget.dropGadget("storage")
            .push(function () {}, function () {})
            .push(function () {
              return gadget.declareGadget(
                url,
                {
                  "scope": "storage",
                  "sandbox": "iframe",
                  "element": gadget.props.element.querySelector('div')
                });
            });
        }
      })
      .push(undefined, function (e) {
        // Ugly Hack to reload page and make service worker available
        if (e &&
            e.indexOf("Please reload this page to allow Service Worker to control this page") > -1) {
          console.log("reload");
          window.location.reload(false);
          throw (e);
        } else {
          console.log(e);
          throw (e);
        }
      });
  }

  function isValidUrl(string) {
    try {
      new URL(string);
    } catch (_) {
      return false;
    }
    return true;
  }

  rJS(window)
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          var getURL = window.location;
          g.props.element = element;
        });
    })
    .ready(function (gadget) {
      // Initialize the gadget local parameters
      gadget.state_parameter_dict = {};
      return getStorageGadget(gadget);
    })
    .declareAcquiredMethod("getSetting", "getSetting")
    .declareAcquiredMethod("setSetting", "setSetting")
    .declareMethod('allDocs', function (params) {
      return getStorageGadget(this)
        .push(function (storage_gadget) {
          return storage_gadget.allDocs(params);
        });
    })

    .declareMethod('get', function (url) {
      return getStorageGadget(this)
        .push(function (storage_gadget) {
          return storage_gadget.get(url);
        })
        .push(function (result) {
          return jIO.util.dataURItoBlob(result);
        });
    })

    .declareMethod('put', function (url, parameter) {
      var blob, gadget = this;
      if (parameter.blob !== undefined) {
        blob = parameter.blob;
      } else {
        blob = new Blob(
          [parameter.content],
          {type: parameter.type}
        );
      }
      return RSVP.Queue()
        .push(function () {
          return RSVP.all([
            getStorageGadget(gadget),
            jIO.util.readBlobAsDataURL(blob)
          ]);
        })
        .push(function (result_list) {
          return result_list[0].put(url, result_list[1].target.result);
        })
        .push(console.log, console.log)
        ;
    })

    .declareMethod('remove', function (url) {
      return getStorageGadget(this)
        .push(function (storage_gadget) {
          return storage_gadget.remove(url);
        });
    })
    .declareMethod('getScope', function () {
      return getStorageGadget(this)
        .push(function (storage_gadget) {
          return storage_gadget.getScope();
        });
    })
    .declareMethod('setCribEnableGadgetUrl', function (url) {
      var gadget = this;
      if (!isValidUrl(url)) {
        url = new URL(url, window.location.href).href;
      }
      return new RSVP.Queue()
        .push(function () {
          return gadget.getDeclaredGadget("crib_enable_validator");
        })
        .push(function (validator_gadget) {
          return validator_gadget.validateCribEnableGadgetUrl(url);
        })
        .push(function (result) {
          if (!result) {
            throw "Could not Validate CribJS Gadget URL: " + url;
          }
          return gadget.setSetting("site_editor_gadget_url", url);
        })
        .push(function () {
          // XXX CLN Needs to call that one here in order to make sure we do not
          // run accidently in a possible bug in RenderJS when the same gadget
          // is declared twice in parallel. CLN needs to reproduce and
          // investigate this issue.
          return getStorageGadget(gadget, url);
        });
    })
    .declareMethod('getCribEnableGadgetUrl', function (url) {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.storage_gadget_url;
        });
    });

}(window, rJS, RSVP, jIO));