/*jslint browser: true, indent: 2, maxlen: 80*/
/*globals RSVP, rJS,
         location, console, fetch, Promise*/
(function (window, document, RSVP, rJS, location, console, JSZip, mimeType) {
  "use strict";

  function loadZipIntoCrib(gadget, zip, from_path, path_to_load, logFunction) {
    if (logFunction === undefined) {
      logFunction = console.log;
    }
    var promise_list = [], url_number = 0, path_list = [], root_path = "", i;
    zip.forEach(function (relativePath, zipEntry) {
      path_list.push(relativePath);
    });
    if (path_list && path_list.length > 1) {
      root_path = path_list[0].split("/")[0];
      for (i = 0; i < path_list.length - 1; i++) {
        if (root_path !== path_list[i].split("/")[0]) {
          root_path = "";
          break;
        }
      }
    }
    if (root_path !== "" && !from_path) {
      from_path = root_path;
    }
    zip.forEach(function (relativePath, zipEntry) {
      var end_url;
      url_number += 1;
      if (zipEntry.dir) {
        return;
      }
      if (!relativePath.startsWith(from_path)) {
        return;
      }
      relativePath = relativePath.substring(from_path.length);
      if (path_to_load && !relativePath.startsWith("/") &&
          !path_to_load.endsWith("/")) {
        end_url = path_to_load + "/" + relativePath;
      } else if (relativePath.startsWith("/") &&
                 (!path_to_load || path_to_load.endsWith("/"))) {
        end_url = path_to_load + relativePath.substring(1);
      } else {
        end_url = path_to_load + relativePath;
      }
      promise_list.push(
        new RSVP.Queue()
        .push(function () {
          return zipEntry.async('blob');
        })
        .push(function (result) {
          var guessed_type = mimeType.lookup(
            end_url, false, "application/octet-stream"
          );
          result = result.slice(0, result.size, guessed_type);
          return gadget.crib_sw_put(end_url, {blob: result});
        })
      );
    });
    logFunction("Pushing " + promise_list.length + " URLs to local crib");
    return RSVP.all(promise_list);
  }

  function loadContentFromZIPFile(gadget, options, logFunction) {
    if (logFunction === undefined) {
      logFunction = console.log;
    }
    var path_to_load = options.to_path,
      from_path = options.from_path,
      file_list = options.file_list;
    if (file_list.length === 0) {
      logFunction("No File to Load");
      return "No File to Load";
    }
    return new RSVP.Queue()
    .push(function () {
      logFunction("Loading Zip");
      return JSZip.loadAsync(file_list[0]);
    })
    .push(function (zip) {
      return loadZipIntoCrib(
        gadget, zip, from_path, path_to_load, logFunction
      );
    })
    .push(function () {
      logFunction("Imported Zip Successfully");
    }, function (e) {
      logFunction(e);
    });
  }


  function loadContentFromZIPURL(gadget, options, logFunction) {
    if (logFunction === undefined) {
      logFunction = console.log;
    }
    var path_to_load = options.to_path, file_list,
      from_path = options.from_path, zip_url = options.zip_url,
      jio_gadget, url_list = [], url_number = 0;
    return new RSVP.Queue()
    .push(function () {
      logFunction("Fetching Url");
      return fetch(zip_url)
        .then(function (response) {                       // 2) filter on 200 OK
            if (response.status === 200 || response.status === 0) {
              logFunction("Fetched Successfully, Starts Loading Zip");
              return Promise.resolve(response.blob());
            } else {
              logFunction("ERROR: " + response.statusText);
              return Promise.reject(new Error(response.statusText));
            }
          });
    })
    .push(JSZip.loadAsync)
    .push(function (zip) {
      return loadZipIntoCrib(gadget, zip, from_path, path_to_load, logFunction);
    })
    .push(function () {
      logFunction("Imported Zip Successfully");
    }, function (e) {
      logFunction(e);
      throw e;
    });
  }

  rJS(window)
    .ready(function (g) {
    g.props = {};
  })
    .declareMethod('loadFromZipUrl', function (options, logFunction) {
      return loadContentFromZIPURL(this, options, logFunction);
    })
    .declareMethod('loadFromZipFile', function (options, logFunction) {
      return loadContentFromZIPFile(this, options, logFunction);
    })
    .declareAcquiredMethod("crib_sw_put", "crib_sw_put")
    .declareAcquiredMethod("getSetting", "getSetting")
    .allowPublicAcquisition("getSetting", function (argument_list) {
      return this.getSetting(argument_list[0], argument_list[1]);
    });

}(window, document, RSVP, rJS, location, console, JSZip, mimeType));